<?php

if ( ! defined( 'ABSPATH' ) ) exit;

class WooCommerce_Shipmate_Order_Actions {

	/**
	 * The single instance of WooCommerce_Shipmate_Order_Actions.
	 * @var 	object
	 * @access  private
	 * @since 	1.0.0
	 */
	private static $_instance = null;

	/**
	 * The main plugin object.
	 * @var 	object
	 * @access  public
	 * @since 	1.0.0
	 */
	public $parent = null;

	/**
	 * Prefix for plugin settings.
	 * @var     string
	 * @access  public
	 * @since   1.0.0
	 */
	public $base = '';



	/**
	 * The shipmate integration class
	 * @var     WooCommerce_Shipmate_Integration
	 * @access  protected
	 * @since   1.0.0
	 */
	protected $shipmate_integration = null;

	/**
	 * The shipmate service class
	 * @var     WooCommerce_Shipmate_Service
	 * @access  protected
	 * @since   1.0.0
	 */
	protected $shipmate_service = null;

	/**
	 * The key for post_meta to state an order has been printed
	 * @var     WooCommerce_Shipmate_Service
	 * @access  protected
	 * @since   1.0.0
	 */
	protected $printed_meta_key = "_wc_order_printed_with_shipmate";
	protected $tracking_reference_meta_key = "_wc_order_tracking_reference";

	/**
	 * Available settings for plugin.
	 * @var     array
	 * @access  public
	 * @since   1.0.0
	 */
	public $settings = array();

	public function __construct ( $parent ) {
		$this->parent = $parent;

		$this->base = 'wpt_';

		$this->shipmate_integration = new WooCommerce_Shipmate_Integration();
		$this->shipmate_service = new WooCommerce_Shipmate_Service($this->shipmate_integration);

		if(!empty(get_option('shipmate_sender_token', null))) {
			// if integrated - we need more tabs!
			add_action( 'woocommerce_order_actions', array( $this, 'shipmate_add_order_meta_box_action' ) );
			add_action( 'woocommerce_order_action_wc_order_print_label', array( $this, 'shipmate_process_order_meta_box_action' ) );
			add_action( 'woocommerce_order_action_wc_order_cancel_label', array( $this, 'shipmate_cancel_order_meta_box_action' ) );
		}
	}

	/**
	 * Add a custom action to order actions select box on edit order page
	 * Only added for paid orders that haven't fired this action yet
	 *
	 * @param array $actions order actions array to display
	 * @return array - updated actions
	 */
	function shipmate_add_order_meta_box_action( $actions ) {
		global $theorder;
		
		if(WooCommerce_Shipmate_Integration::version_check('3.0')) {
			$order_id = $theorder->get_id();
		} else {
			$order_id = $theorder->id;
		}

	    // bail if the order has been paid for or this action has been run
	    if ( ! $this->shipmate_integration->is_integrated() || ! $theorder->is_paid() ) {
	        return $actions;
	    }

	    // add print custom action

	    if(get_post_meta( $order_id, $this->printed_meta_key, true ) == 'yes') {
		    $actions['wc_order_print_label'] = __( 'Reprint Label with Shipmate', 'shipmate' );
		    $actions['wc_order_cancel_label'] = __( 'Cancel Label with Shipmate', 'shipmate' );
		} else {
			$actions['wc_order_print_label'] = __( 'Print Label with Shipmate', 'shipmate' );
		}

	    return $actions;
	}

	/**
	 * Add an order note when custom action is clicked
	 * Add a flag on the order to show it's been run
	 *
	 * @param \WC_Order $order
	 */
	function shipmate_process_order_meta_box_action($order ) {

		if(WooCommerce_Shipmate_Integration::version_check('3.0')) {
			$order_id = $order->get_id();
		} else {
			$order_id = $order->id;
		}



		
	    
		// attempt to create the parcel inside of Shipmate!
		
		$consignment = new WooCommerce_Shipmate_Consignment($order, 
			!empty($_POST['parcel_dimensions']) ? $_POST['parcel_dimensions'] : array(), 
			!empty($_POST['delivery_service_key']) ? $_POST['delivery_service_key'] : "",
			!empty($_POST['dispatch_date']) ? $_POST['dispatch_date'] : date('Y-m-d', strtotime('now'))
		);
		// $consignment->set_delivery_service_key(!empty($_POST['delivery_service_key']) ? $_POST['delivery_service_key'] : "");

		$wordpress_user = wp_get_current_user();
		$shipmate_user_id = get_user_meta($wordpress_user->ID, '_shipmate_user_id', true);
		$result = $this->shipmate_service->create_consignment($shipmate_user_id, $consignment->toArray());
		
		switch($result->http_code()) {
			case "403":
				$message = sprintf( __( 'Unable to connect to Shipmate (Invalid Access Token).', 'shipmate' ));
	    		$order->add_order_note( $message );
	    		break;
	    	case "400":

	    		$message = sprintf( __( 'Unable to create Consignment with Shipmate (Missing Data).', 'shipmate' ));
	    		$order->add_order_note( $message );



	    		if(get_option("_shipmate_status_failed", "Don't Change") != "Don't Change") {

					if(WooCommerce_Shipmate_Integration::version_check('3.0')) {
						$order->set_status(get_option("_shipmate_status_failed"));
						$order->save();
					} else {
						$order->update_status(get_option("_shipmate_status_failed"));
					}

					
				}

	    		break;
	    	case "201":
			    foreach($result->data() as $key => $label) {
			    	// if we haven't previously created these parcels, save the tracking reference and dimensions
			    	if(get_post_meta( $order_id, $this->printed_meta_key, true ) != 'yes') {
				    	$message = sprintf( __( 'Shipping Label generated with Tracking Reference %s for %s. (%s - %s)', 'shipmate' ), $label->tracking_reference, $label->parcel_reference, $label->carrier, $label->service_name );
				    	$order->add_order_note( $message );


				    	

				    	// store the carrier and service used
				    	update_post_meta( $order_id, '_shipmate_carrier', $label->carrier );
				    	update_post_meta( $order_id, '_shipmate_service', $label->service_name );

				    	// store tracking references / dims in meta for the order
				    	add_post_meta($order_id, '_shipmate_parcel_reference', $label->parcel_reference, false);
				    	add_post_meta($order_id, '_shipmate_tracking_reference', $label->tracking_reference, false);
				    	add_post_meta($order_id, '_shipmate_parcel_weight', $_POST['parcel_dimensions']['parcel_weights'][$key], false);
				    	add_post_meta($order_id, '_shipmate_parcel_length', $_POST['parcel_dimensions']['parcel_lengths'][$key], false);
				    	add_post_meta($order_id, '_shipmate_parcel_width', $_POST['parcel_dimensions']['parcel_widths'][$key], false);
				    	add_post_meta($order_id, '_shipmate_parcel_depth', $_POST['parcel_dimensions']['parcel_depths'][$key], false);

				    	if ( function_exists( 'wc_st_add_tracking_number' ) ) {
				    		$provider = $this->shipmate_integration->convert_carrier_to_provider($label->carrier);
				    		
							wc_st_add_tracking_number( $order_id, $label->tracking_reference, $provider, null, null );
						}

						if(get_option("_shipmate_status_success", "Don't Change") != "Don't Change") {
							if(WooCommerce_Shipmate_Integration::version_check('3.0')) {
								$order->set_status(get_option("_shipmate_status_success"));
								$order->save();
							} else {
								$order->update_status(get_option("_shipmate_status_success"));
							}
							
						
						}
				    }
			    }
	    		
			    update_post_meta( $order_id, $this->printed_meta_key, 'yes' );
			    $message = sprintf( __( 'Shipping Label' . (count($result->data()) == 1 ? '' : 's') . ' printed with Shipmate by %s.', 'shipmate' ), wp_get_current_user()->display_name );
			    $order->add_order_note( $message );
	    		break;
	    	default:
	    		die(var_dump($result));
	    		break;
		}
	}

	/**
	 * Cancel an Order in Shipmate
	 * Add a flag on the order to show it's been run
	 *
	 * @param \WC_Order $order
	 */
	function shipmate_cancel_order_meta_box_action( $order ) {

		if(WooCommerce_Shipmate_Integration::version_check('3.0')) {
			$order_id = $order->get_id();
		} else {
			$order_id = $order->id;
		}
	    
		// TODO: Cancel Consignment via service...
		$consignment = new WooCommerce_Shipmate_Consignment($order, array());

		$result = $this->shipmate_service->cancel_consignment($consignment->sender_reference());
		//Either 200 or 404 as they can cancel the parcel in Shipmate we will still need to remove the information
		if($result->http_code() == 200 || $result->http_code() == 404) {

		    // translators: Placeholders: %s is a user's display name
		    $message = sprintf( __( 'Consignment cancelled with Shipmate by %s.', 'shipmate' ), wp_get_current_user()->display_name );
		    $order->add_order_note( $message );
		    
		    // add the flag
		    update_post_meta( $order_id, $this->printed_meta_key, 'no' );

	    	delete_post_meta( $order_id, '_shipmate_carrier');
	    	delete_post_meta( $order_id, '_shipmate_service');

	    	// store tracking references / dims in meta for the order
	    	delete_post_meta($order_id, '_shipmate_parcel_reference');
	    	delete_post_meta($order_id, '_shipmate_tracking_reference');
	    	delete_post_meta($order_id, '_shipmate_parcel_weight');
	    	delete_post_meta($order_id, '_shipmate_parcel_length');
	    	delete_post_meta($order_id, '_shipmate_parcel_width');
	    	delete_post_meta($order_id, '_shipmate_parcel_depth');


	    	if(get_option("_shipmate_status_cancel", "Don't Change") != "Don't Change") {

				if(WooCommerce_Shipmate_Integration::version_check('3.0')) {
					$order->set_status(get_option("_shipmate_status_cancel"));
					$order->save();
				} else {
					$order->update_status(get_option("_shipmate_status_cancel"));
				}
				
			}

			return true;
		} 

		die(var_dump($result));
	}

	/**
	 * Main WooCommerce_Shipmate_Order_Actions Instance
	 *
	 * Ensures only one instance of WooCommerce_Shipmate_Order_Actions is loaded or can be loaded.
	 *
	 * @since 1.0.0
	 * @static
	 * @see WooCommerce_Shipmate()
	 * @return Main WooCommerce_Shipmate_Order_Actions instance
	 */
	public static function instance ( $parent ) {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self( $parent );
		}
		return self::$_instance;
	} // End instance()

	/**
	 * Cloning is forbidden.
	 *
	 * @since 1.0.0
	 */
	public function __clone () {
		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?' ), $this->parent->_version );
	} // End __clone()

	/**
	 * Unserializing instances of this class is forbidden.
	 *
	 * @since 1.0.0
	 */
	public function __wakeup () {
		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?' ), $this->parent->_version );
	} // End __wakeup()

}
